class Genetic:
    """
    A Base class for genetical objects,
    all the instances can inherite attributes from its parent. 
    """
    def __init__(self):
        self.__parent__ = None
        self.__children__ = []
        #print dir(self)

    def __getattr__(self,n):
        d = self.__dict__
        if n in d.keys():
            return d[n]
        p = self.__parent__ #d['__parent__']
        if p is not None:
            return getattr(p,n)
        raise AttributeError(n)

    def addChild(self,c):
        self.__children__.append(c)
        c.__parent__ = self
           
    def setParent(self,p):
        p.addChild(self)

if __name__ == '__main__':
    class A(Genetic):
        def __init__(self,a1,a2):
            Genetic.__init__(self)
            self.a1 = a1
            self.a2 = a2
        
    class B(Genetic):
        def __init__(self,b):
            Genetic.__init__(self)
            self.b = b
    gp = A(1,2)
    p = B(3)
    c = A(4,5)
    gc = B(6)
    gc.setParent(c)
    c.setParent(p)
    p.setParent(gp)



import asyncio
import aiohttp
import re

RESPONSE_BIN = 0
RESPONSE_TEXT = 1
RESPONSE_JSON = 2
RESPONSE_FILE = 3

class HttpClient:
	def __init__(self,coding='utf-8'):
		self.coding = coding
		self.session = None
		self.cookies = {}

	async def close(self):
		await self.session.close()

	def url2domain(self,url):
		parts = url.split('/')[:3]
		pre = '/'.join(parts)
		return pre
		
	def setCookie(self,url,cookies):
		name = self.url2domain(url)
		self.cookies[name] = cookies

	def getCookies(self,url):
		name = url2domain(url)
		return self.cookies.get(name,None)

	def getsession(self,url):
		if self.session is None:
			jar = aiohttp.CookieJar(unsafe=True)
			self.session = aiohttp.ClientSession(cookie_jar=jar)
		return self.session
				
	async def handleResp(self,url,resp,resp_type):
		if resp.cookies is not None:
			self.setCookie(url,resp.cookies)

		if resp_type == RESPONSE_BIN:
			return await resp.read()
		if resp_type == RESPONSE_JSON:
			return await resp.json()
		# default resp_type == RESPONSE_TEXT:
		return await resp.text(self.coding)

	def grapCookie(self,url):
		session = self.getsession(url)
		domain = self.url2domain(url)
		filtered = session.cookie_jar.filter_cookies(domain)
		return filtered
		print(f'=====domain={domain},cookies={fltered},type={type(fltered)}===')

	async def get(self,url,
					params={},headers={},resp_type=RESPONSE_TEXT):
		session = self.getsession(url)
		resp = await session.get(url,params=params,headers=headers)
		if resp.status==200:
			return await self.handleResp(url,resp,resp_type)
		return None

	async def post(self,url,
					data=b'',headers={},cookies=None,
					resp_type=RESPONSE_TEXT):
		session = self.getsession(url)
		resp = await session.post(url,data=data,
						headers=headers,cookies=cookies)
		if resp.status==200:
			return await self.handleResp(url,resp,resp_type)
		print(f'**ERROR**{url} response code={resp.status}')
		return None

if __name__ == '__main__':
	async def gbaidu(hc):
		r = await hc.get('https://www.baidu.com')
		print(r)
		await hc.close()
	loop = asyncio.get_event_loop()
	hc = HttpClient()
	loop.run_until_complete(gbaidu(hc))




import time
from ffpyplayer.player import  MediaPlayer
from ffpyplayer.tools import set_log_callback, get_log_callback, formats_in

class AudioPlayer:
	def __init__(self, source=None, autoplay=False, 
				loop=False, 
				on_stop=None):
		self.volume = 1
		self.state = None
		self.source = None
		self.quitted = False
		self.loop = loop
		self.autoplay = autoplay
		self.player = None
		self.on_stop = on_stop
		self.cmds = []
		if source:
			self.set_source(source)

	def set_source(self, source):
		self.source = source
		self.load()

	def player_callback(self, selector, value):
		print(f'player_callback(): {selector}, {value}')
		if self.player is None:
			print(f'player_callback():self.player is None, {selector}, {value}')
			return
		if selector == 'quit':
			def close(*args):
				self.quitted = True
				self.unload()
			close()
		
		elif selector == 'eof':
			self._do_eos()

	def load(self):
		if self.source is None:
			return
		source = self.source
		self.unload()
		ff_opts = {'vn':True, 'sn':True}
		self.player = MediaPlayer(source,
			callback=self.player_callback,
			loglevel='info',
			ff_opts=ff_opts)
		player = self.player
		self.state = 'pause'
		s = time.perf_counter()
		while (player.get_metadata()['duration'] is None and
				not self.quitted and 
				time.perf_counter() - s < 10.):
			time.sleep(0.005)
		player.toggle_pause()
		time.sleep(0.04)
		player.set_volume(self.volume)
		if self.autoplay:
			self.play()

	def unload(self):
		self.player = None
		self.state = 'stop'
		self.quitted = False

	def __del__(self):
		self.unload()

	def play(self):
		if self.player is None:
			self.load()
		if self.player is None:
			print('play():self.player is None')
			return
		if self.state == 'play':
			return
		self.player.toggle_pause()
		self.state = 'play'

	def pause(self):
		if self.player is None:
			self.load()
		if self.player is None:
			print('pause():self.player is None')
			return
		if self.state == 'pause':
			return
		self.player.toggle_pause()
		self.state = 'pause'

	def is_busy(self):
		if self.player and self.state == 'play':
			return True
		return False

	def stop(self):
		if self.player is None:
			return
		if self.state == 'play':
			self.player.toggle_pause()
		self.state = 'stop'
		self.seek(0)
		if self.on_stop:
			self.on_stop()
	
	def seek(self, pos):
		if self.player is None:
			print('seek():self.player is None')
			return
		self.player.seek(pos, relative=False)

	def get_pos(self):
		if self.player is None:
			return 0
		return self.player.get_pts()

	def _do_eos(self, *args):
		print('_do_eos() called ...')
		if self.loop:
			self.seek(0.)
		else:
			print('go stop')
			self.stop()


if __name__ == '__main__':
	import sys
	p = AudioPlayer(autoplay=True, loop=True)
	p.source = sys.argv[1]
	p.load()
	p.play()
	while True:
		while p._pump():
			pass

		print("""
play: play it,
stop: stop play
pause:pause it
quit: exit
""")
		x = input()
		if x == 'quit':
			p.quitted = True
			p.stop()
			break
		if x == 'play':
			p.play()
			continue
		if x == 'stop':
			p.stop()
			continue
		if x == 'pause':
			p.pause()
			continue




# !/usr/bin/env python
#
# ObjectsCache is a Objects cache
# the Object has to have a method "get_size" to tell 
#   the cacher counting the objects size

class ObjectCache(dict) :
	def __init__(self,maxsize=10000000,*args) :
		super(ObjectsCache,self).__init__(*args) 
		self.maxsize = maxsize
		self.size = 0
		self._shadow = {}

	def __setitem__(self,key,item) :
		try :
			size = item.get_size()
			self.size += size
		except :
			return
		if self.size >= self.maxsize :
			tmp = [(t,key) for key,(t,size) in self._shadow.iteritems() ]
			tmp.sort()
			for i in xrange(len(tmp)//2) :
				del self[tmp[i][i]]
			del tmp
		super(ObjectCache,self).__setitem__(key,item)
		self._shadow[key] = [time.time(),size]

	def __getitem__(self,key) :
		try :
			item = super(ObjectCache,self).__getitem__(key)
		except :
			raise
		else :
			self._shadow[key][0] = time.time()
			return item

	def get(self,key,default=None) :
		if self.has_key(key) :
			return self[key]
		else :
			return default

	def __delitem__(self,key) :
		try :
			super(ObjectCache,self).__delitem__(key)
		except :
			raise
		else :
			self.size -= self._shadow[key][1]
			del self._shadow[key]





# this function will fix a bug for open a file with a not english name.
#

import sys
import locale

language, local_encoding = locale.getdefaultlocale()
if sys.platform == 'win32':
     import locale, codecs
     local_encoding = locale.getdefaultlocale()[1]
     if local_encoding.startswith('cp'):            # "cp***" ?
         try:
             codecs.lookup(local_encoding)
         except LookupError:
             import encodings
             encodings._cache[local_encoding] = encodings._unknown
             encodings.aliases.aliases[local_encoding] = 'mbcs'
    
def locale_open(filename,mode='rb') :
    return open(filename.encode(local_encoding),mode)

def localeString(s) :
	try :
		return unicode(s,'utf-8').encode(local_encoding)
		
	except :
		return s

def utf8String(s) :
	try :
		return unicode(s,local_encoding).encode('utf-8')
	except :
		return s

def charsetString(s,charset) :
	try :
		return unicode(s,local_encoding).encode(charset)
	except :
		try :
			return unicode(s,'utf-8').encode(charset)
		except :
			return s




try:
    import ujson as json
except:
    import json

import codecs

def loadf(fn,coding='utf8'):
	f = codecs.open(fn,'r',coding)
	d = json.load(f)
	f.close()
	return d
	
def dumpf(obj,fn,coding='utf8'):
	f = codecs.open(fn,'w',coding)
	json.dump(obj,f)
	f.close()

load = json.load
dump = json.dump
loads = json.loads
dumps = json.dumps



#unidict.py
import locale

def unicoding(d,coding='utf8'):
	if type(d) == type(''):
		return d
	if type(d) == type(b''):
		try:
			if coding is not Noene:
				return d.decode(coding)
			else:
				return d.decode(locale.getdefaultlocale()[1])
		except:
			try:
				return d.decode(locale.getdefaultlocale()[1])
			except:
				try:
					return d.decode('utf8')
				except:
					return d
	return d

def uObject(obj,coding='utf8'):
	otype = type(obj)
	if otype == type(u''):
		return obj
	if otype == type({}):
		return uDict(obj,coding)
	if otype == type([]):
		return [uObject(i,coding) for i in obj ]
	if hasattr(obj,'decode'):
		return obj.decode(coding)
	return obj
	
def uDict(dict,coding='utf8'):
	d = {}
	for k,v in dict.items():
		d[uObject(k)] = uObject(v)
	return d




def arrayExtend(s,addon):
	ret = []
	s_cnt = len(s)
	a_cnt = len(addon)
	for i,v in enumerate(addon):
		if i < s_cnt:
			if type(v)!=type(s[i]):
				ret.append(v)
				continue
			if isinstance(v,dict):
				x = dictExtend(v,s[i])
				ret.append(x)
				continue
		ret.append(v)
	if s_cnt < a_cnt:
		ret += s[i:]
	return ret

def dictExtend(s,addon):
	ret = {}
	ret.update(s)
	skeys = ret.keys()
	for k,v in addon.items():
		if k not in skeys:
			ret[k] = v
			continue
		if type(v)!=type(ret[k]):
			ret[k] = v
			continue
		if type(v)==type({}):
			ret[k] = dictExtend(ret[k],v)
			continue
		if type(v)==type([]):
			ret[k] = arrayExtend(ret[k],v)
			continue
		ret[k] = v
	return ret



import os
import sys
from requests import get
from bs4 import BeautifulSoup
from appPublic.http_client import Http_Client
from appPublic.sockPackage import get_free_local_addr
public_headers = {
	"User-Agent":"Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/60.0.3100.0 Safari/537.36"
}

def get_outip():
	ip = get('https://api.ipify.org').content.decode('utf8')
	return ip

def ipip(ip=None):
	# ipip.net
	if ip is None:
		ip = get_outip()
	api= f"http://freeapi.ipip.net/{ip}"
	hc = Http_Client()
	r= hc.get(api, headers=public_headers)
	return {
		'country':r[0],
		'city':r[2]
	}

def ipapi_co(ip):
	url = f'https://ipapi.co/{ip}/json/'
	hc = Http_Client()
	r = hc.get(url)
	r['City'] = r['city']
	r['lat'] = r['latitude']
	r['lon'] = r['longitude']
	return r

def ip_api_com(ip):
	url = f'http://ip-api.com/json/{ip}'
	hc = Http_Client()
	r = hc.get(url)
	r['City'] = r['city']
	return r

def iplocation(ip=None):
	if ip is None:
		ip = get_outip()
	# apikey come from
	# https://app.apiary.io/globaliptv/tests/runs
	# using my github accout
	apikey='c675f89c4a0e9315437a1a5edca9b92c'
	api = f"https://www.iplocate.io/api/lookup/{ip}?apikey={apikey}",
	hc = Http_Client()
	r= hc.get(api, headers=public_headers)
	return r

def get_ip_location(ip):
	funcs = [
		ip_api_com,
		ipapi_co,
		ipip,
		iplocation
	]
	hc = Http_Client()
	for f in funcs:
		try:
			r = f(ip)
			return r
		except:
			pass
		
if __name__ == '__main__':
	print(get_free_local_addr())
	if len(sys.argv) > 1:
		info = get_ip_location(sys.argv[1])
		print(info)




import os
from datetime import datetime
from PublicData import public_data
from folderUtils import mkdir

myLogPath  = '.'

AllCatelogs=['SYSError',
	'SYSWarn',
	'APPError',
	'APPWarn',
	'APPInfo',
	'DEBUG1',
	'DEBUG2',
	'DEBUG3',
	'DEBUG4',
	'DEBUG5',
]

class MyLog :
	def __init__(self,path) :
		self.setLogPath(path)

	def setLogPath(self,path='.') :
		self.myLogPath = path
	logp=os.path.join(path,'log')
	mkdir(logp)

	def __call__(self,msg='') :
		p = os.path.join(self.myLogPath,'log','my.log')
		f = open(p,'a')
		d = datetime.now()
		f.write('%04d-%02d-%02d %02d:%02d:%02d %s\n' % ( d.year,d.month,d.day,d.hour,d.minute,d.second,msg))
		f.close()

class LogMan :
	def __init__(self) :
		self.logers = {}
		self.catelogs = AllCatelogs
	
	def addCatelog(self,catelog) :
		if catelog not in self.catelogs :
			self.catelogs.append(catelog)
		
	def addLoger(self,name,func,catelog) :
		if type(catelog)!=type([]) :
			catelog = [catelog]
		catelog = [ i for i in catelog if i in self.catelogs ]
		log = {
			'name':name,
			'func':func,
			'catelog':catelog,
			}
		self.logers[name] = log
	
	def delLoger(self,name) :
		if name in self.logers.keys() :
			del self.logers[name]
			
	def setCatelog(self,name,catelog) :
		if type(catelog)!=type([]) :
			catelog = [catelog]
		catelog = [ i for i in catelog if i in self.catelogs ]
		if name in self.logers.keys() :
			log = self.logers[name]
			log['catelog'] = catelog
			self.logers[name] = log
	
	def __call__(self,msg='',catelog='APPInfo') :
		for name,loger in self.logers.items() :
			c = loger['catelog']
			if type(c)!=type([]) :
				c = [c]
			if catelog in c :
				f = loger['func']
				f(msg)
				
def mylog(s,catelog='APPInfo') :
	logman = public_data.get('mylog',None)
	if logman==None :
		path = public_data.get('ProgramPath',None)
		if path==None :
			raise Exception('ProgramPath Not found in "public_data"')
		log = MyLog(path)
		logman = LogMan()
		logman.addLoger('mylog',log,AllCatelogs)
		public_data.set('mylog',logman)
	return logman(s,catelog)




import socket
import socks
import requests

original_socket = socket.socket

def set_socks_proxy(host, port):
	socks.setdefaultproxy(socks.PROXY_TYPE_SOCKS5, host, port)
	socket.socket = socks.socksocket

def unset_proxy():
	socket.socket = original_socket



from appPublic.Singleton import SingletonDecorator

@SingletonDecorator
class ObjectAction(object):
	def __init__(self):
		self.actionList = {}
	
	def init(self,id,action):
		idA = self.actionList.get(id,None)
		if idA is None:
			idA = self.actionList[id] = {}
		self.actionList[id][action] = []
		
	def add(self,id,action,func):
		idA = self.actionList.get(id,None)
		if idA is None:
			idA = self.actionList[id] = {}
		fL = idA.get(action,None)
		if fL is None:
			fL = self.actionList[id][action] = []
		self.actionList[id][action].append(func)

	def execute(self,id,action,data,callback=None):
		if action in ['#','*']:
			return data
		idA = self.actionList.get(id,None)
		if idA is None:
			return data
		fL = idA.get(action,[])
		fL += idA.get('*',[])
		for f in fL:
			data = f(id,action,data)
		if len(fL)==0:
			for f in idA.get('#',[]):
				data = f(id,action,data)
		if callback is not None:
			callback(data)
		return data

if __name__ == '__main__':
	def f(id,act,data):
		return data

	def f1(id,act,data):
		return data

	def f2(id,act,data):
		return data
		
	def add():
		oa = ObjectAction()
		oa.add('test','b',f)
		#oa.add('test','*',f1)
		oa.add('test','#',f2)

	def exe():
		oa = ObjectAction()
		oa.execute('test','a','data1')

	add()
	exe()
	



import random
import asyncio
from functools import wraps

def asyncCall(func):
	@wraps(func)
	def wraped_func(*args,**kw):
		task =  asyncio.ensure_future(func(*args,**kw))
		asyncio.gather(task)
	return wraped_func

class Worker:
	def __init__(self,maxtask=50):
		self.semaphore = asyncio.Semaphore(maxtask)

	async def __call__(self,callee,*args,**kw):
		async with self.semaphore:
			return await callee(*args,**kw)

	async def run(self,cmd):
		async with self.semaphore:
			proc = await asyncio.create_subprocess_shell(cmd,
				stdout=asyncio.subprocess.PIPE,
				stderr=asyncio.subprocess.PIPE)

			stdout, stderr = await proc.comunicate()
			return stdout, stderr

if __name__ == '__main__':
	async def hello(cnt,greeting):
		t = random.randint(1,10)
		await asyncio.sleep(t)
		print(cnt,'cost ',t,'seconds to',greeting)

	
	async def run():
		w = Worker()
		tasks = [ w(hello,i,'hello world') for i in range(1000) ]
		await asyncio.wait(tasks)

	loop = asyncio.get_event_loop()
	loop.run_until_complete(run())
	



import os,sys
import time
import datetime

leapMonthDays = [0,31,29,31,30,31,30,31,31,30,31,30,31]
unleapMonthDays = [0,31,28,31,30,31,30,31,31,30,31,30,31]

def curDatetime():
	return datetime.datetime.now()

def curDateString():
	d = curDatetime()
	return '%04d-%02d-%02d' %(d.year,d.month,d.day)
	
def curTimeString():
	d = curDatetime()
	return '%02d:%02d:%02d' %(d.hour,d.minute,d.second)
	
def timestampstr():
	d = curDatetime()
	return '%04d-%02d-%02d %02d:%02d:%02d.%03d' % (d.year,
			d.month,
			d.day,
			d.hour,
			d.minute,
			d.second,
			d.microsecond/1000)
	
def isMonthLastDay(d):
	dd = datetime.timedelta(1)
	d1 = d + dd
	if d1.month != d.month:
		return True
	return False

def isLeapYear(year):
	if year % 4 == 0 and year % 100 == 0 and not (year % 400 == 0):
		return True
	return False
		
def timestamp(dt):
	return int(time.mktime((dt.year,dt.month,dt.day,dt.hour,dt.minute,dt.second,dt.microsecond,0,0)))

def timeStampSecond(dt):
	return int(time.mktime((dt.year,dt.month,dt.day,dt.hour,dt.minute,dt.second,0,0,0)))

def addSeconds(dt,s):
	ndt = dt + datetime.timedelta(0,s)
	return ndt
	
def monthMaxDay(y,m):
	if isLeapYear(y):
		return leapMonthDays[m]
	return unleapMonthDays[m]

def date2str(dt=None):
	if dt is None:
		dt = curDatetime()
	return '%04d-%02d-%02d' % (dt.year,dt.month,dt.day)

def time2str(dt):
	return '%02d:%02d:%02d' % (dt.hour,dt,minute,dt.second)
	
def str2Date(dstr):
	try:
		haha = dstr.split(' ')
		y,m,d = haha[0].split('-')
		H = M = S = 0
		if len(haha) > 1:
			H,M,S = haha[1].split(':')
		return ymdDate(int(y),int(m),int(d),int(H),int(M),int(S))
	except Exception as e:
		print(e)
		return None
		
def ymdDate(y,m,d,H=0,M=0,S=0):
	return datetime.datetime(y,m,d,H,M,S)
	
def str2Datetime(dstr):
	x = dstr.split(' ')
	d = x[0]
	t = '00:00:00'
	if len(x) > 1:
		t = x[1]
	y,m,d = d.split('-')
	H,M,S = t.split(':')
	return datetime.datetime(int(y),int(m),int(d),int(H),int(M),int(S))
	
def strdate_add(date_str, days=0, months=0, years=0):
	dt = str2Datetime(date_str)
	dt = dateAdd(dt, days=days, months=months, years=years)
	ds = date2str(dt)
	return ds

def addMonths(dt,months):
	y = dt.year
	m = dt.month + months
	d = dt.day
	mm = (m - 1) % 12 + 1
	md = int((m - 1) / 12)
	y += md
	m = mm
	maxd = monthMaxDay(y,m)
	if d > maxd:
		d = maxd
	return ymdDate(y,m,d)

def addYears(dt,years):
	y = dt.year + years
	m = dt.month
	d = dt.day
	maxd = monthMaxDay(y,m)
	if d > maxd:
		d = maxd
	return ymdDate(y,m,d)

def dateAdd(dt,days=0,months=0,years=0):
	if days != 0:
		dd = datetime.timedelta(days)
		dt = dt + dd	
	if months != 0:
		dt = addMonths(dt,months)
	if years != 0:
		dt = addYears(dt,years)
	return dt

def firstSunday(dt):
	f = dt.weekday()
	if f<6:
		return dt + datetime.timedelta(7 - f)
	return dt

DTFORMAT = '%Y%m%d %H%M%S'
def getCurrentTimeStamp() :
	t = time.localtime()
	return TimeStamp(t)
	
def TimeStamp(t) :
	return time.strftime(DTFORMAT,t)

def StepedTimestamp(baseTs,ts,step) :
	if step<2 :
		return ts
	offs = int(timestampSub(ts,baseTs))
	step = int(step)
	r,m = divmod(offs,step)
	if m < step/2 :
		return timestampAdd(baseTs,step * r)
	else :
		return timestampAdd(baseTs,step * (r+1))
		
def timestampAdd(ts1,ts2) :
	t1 = time.strptime(ts1,DTFORMAT)
	tf = time.mktime(t1)
	if type(ts2)=='' :
		t2 = time.strptime(ts2,DTFORMAT)
		ts2 = time.mktime(t2)
	tf += ts2
	t = time.localtime(tf)
	return TimeStamp(t)

def timestampSub(ts1,ts2) :
	t1 = time.strptime(ts1,DTFORMAT)
	t2 = time.strptime(ts2,DTFORMAT)
	ret = time.mktime(t1) - time.mktime(t2)
	return int(ret)

def timestamp2dt(t):
	return datetime.datetime.fromtimestamp(t)

def date_weekinyear(date_str):
	w = datetime.datetime.strptime(date_str, '%Y-%m-%d').strftime('%W')
	return date_str[:5] + w

def date_season(date_str):
	m = date_str[5:7]
	sl = {
		'01':'1',
		'02':'1',
		'03':'1',
		'04':'2',
		'05':'2',
		'06':'2',
		'07':'3',
		'08':'3',
		'09':'3',
		'10':'4',
		'11':'4',
		'12':'4',
	}
	s = sl.get(m)
	return date_str[:5] + s




try:
	import ujson as json
except:
	import json
from appPublic.rsawrap import RSA
from appPublic.rc4 import RC4
from appPublic.uniqueID import getID

# import brotli
import zlib
import struct

DATA_TYPE_BYTES = 1
DATA_TYPE_STR = 2
DATA_TYPE_JSON = 3

class DataEncoder:
	"""
	security data packing - unpacking object
	packs data:
	encode data with random key's rc4 crypt algorithm, 
	encode rc4's key with receiver's public key
	sign data with sender's private key
	packs data using struct in follows order
	0: data format(18 bytes)
	1. datatype(c)
	2. encoded data(length=len(d))
	3. encoded_rc4key(length=len(k))
	4. sign(signs from (0+1+2+3) data) (length=len(s))
	5. compress data and return compressed dta
	return packed data
	unpacks data:
	0. decompress data
	1. get 18 bytes fmt data, erase tails b'\x00'
	2. using fmt to unpack data[18:]
	3. verify sign
	4. decode k
	5. decode data usig decoded k with rc4 algorithm
	6. convert data type to origin data type
	7. return converted data
	"""
	def __init__(self, myid, func_get_peer_pubkey, private_file=None):
		self.myid = myid
		self.func_get_peer_pubkey = func_get_peer_pubkey
		self.public_keys = {}
		self.private_file = private_file
		self.rsa = RSA()
		self.rc4 = RC4()
		if self.private_file:
			self.private_key = self.rsa.read_privatekey(self.private_file)
		else:
			self.private_key = self.rsa.create_privatekey()
		self.public_key = self.rsa.create_publickey(self.private_key)

	def identify_datatype(self, data):
		if isinstance(data, bytes):
			return DATA_TYPE_BYTES, data
		if isinstance(data, str):
			return DATA_TYPE_STR, data.encode('utf-8')
		data = json.dumps(data).encode('utf-8')
		return DATA_TYPE_JSON, data

	def my_text_publickey(self):
		return self.rsa.publickeyText(self.public_key)

	def exist_peer_publickeys(self, peer_id):
		return True if self.public_keys.get(peer_id, False) else False

	def set_peer_pubkey(self, peer_id, pubkey):
		self.public_keys[peer_id] = pubkey

	def get_peer_text_pubkey(self, peer_id):
		pk = self.get_peer_pubkey()
		txtpk = self.rsa. publickeyText(pk)
		return txtpk

	def set_peer_text_pubkey(self, peer_id, text_pubkey):
		pk = self.rsa.publickeyFromText(text_pubkey)
		self.set_peer_pubkey(peer_id, pk)

	def get_peer_pubkey(self, peer_id):
		pubkey = self.public_keys.get(peer_id)
		if not pubkey:
			try:
				self.func_get_peer_pubkey(peer_id)
			except:
				raise Exception('Can not get peer public key(%s)')
			pubkey = self.public_keys.get(peer_id)
		return pubkey

	def pack(self, peer_id, data, uncrypt=False):
		t, d = self.identify_datatype(data)
		if uncrypt:
			return zlib.compress(b'\x00' * 18 + \
						bytes(chr(t),'utf-8') + \
						d)
		pk = self.get_peer_pubkey(peer_id)
		d, k = self.encode_data(pk, d)
		f = 'b%05ds%03ds' % (len(d), len(k))
		f1 = f + '256s'
		pd1 = struct.pack('18s', f1.encode('utf-8'))
		pd2 = struct.pack(f, t, d, k)
		pd = pd1 + pd2
		s = self.sign_data(pd)
		pd += s
		self.pack_d = [t,d,k,s]
		origin_len = len(pd)
		pd = zlib.compress(pd)
		return pd

	def unpack(self, peer_id, data):
		data = zlib.decompress(data)
		if data[:18] == b'\x00' * 18:
			data = data[18:]
			t = ord(chr(data[0]))
			d = data[1:]
			if t == DATA_TYPE_BYTES:
				return d
			d = d.decode('utf-8')
			if t == DATA_TYPE_STR:
				return d
			return json.loads(d)

		org_data = data
		pk = self.get_peer_pubkey(peer_id)
		f = data[:18]
		while f[-1] == 0 and len(f) > 0:
			f = f[:-1]
		f = f.decode('utf-8')
		data = data[18:]
		t, d, k, s = struct.unpack(f, data)
		self.unpack_d = [t,d,k,s]
		data1 = org_data[:org_data.index(s)]
		if not self.verify_sign(data1, s, pk):
			raise Exception('data sign verify failed')
		data = self.decode_data(d, k)
		if t == DATA_TYPE_BYTES:
			return data
		if t == DATA_TYPE_STR:
			return data.decode('utf-8')
		return json.loads(data)

	def encode_data(self, peer_pubkey, data):
		key = getID()
		if isinstance(key, str):
			key = key.encode('utf-8')
		ctext = self.rc4.encode_bytes(data, key)
		encoded_key = self.rsa.encode_bytes(peer_pubkey, key)
		return ctext, encoded_key

	def sign_data(self, data):
		return self.rsa.sign_bdata(self.private_key, data)

	def decode_data(self, data, encoded_key):
		key = self.rsa.decode_bytes(self.private_key, encoded_key)
		return self.rc4.decode_bytes(data, key)

	def verify_sign(self, data, sign, peer_pubkey):
		return self.rsa.check_sign_bdata(peer_pubkey, data, sign)



#dataMapping
from appPublic.dictObject import DictObject

def keyMapping(dic,mappingtab,keepmiss=True):
	"""
	keyMapping mappingtab
	{
		"a1":"b1",
		"a2":'b2",
		...
	}
	"an" is key in dic
	"bn" is key in result dictionary
	"""
	ret = {}
	keys = [ k for k in dic.keys()]
	if not keepmiss:
		keys = [ k for k in dic.keys() if k in mappingtab.keys() ]
	[ ret.update({mappingtab.get(k,k):dic[k]}) for k in keys ]
	return ret


def valueMapping(dic,mappingtab):
	"""
	mappingtab format:
	{
		"field1":{
			"a":"1",
			"b":"2",
			"__default__":"5"
		},
		"field2":{
			"a":"3",
			"b":"4"
		}
	}
	field1,field2 is in dic.keys()
	"""
	ret = {}
	for k in dic.keys():
		mt = mappingtab.get(k,None)
		if mt is None:
			ret[k] = dic[k]
		else:
			dv = mt.get('__default__',dic[k])
			v = mt.get(dic[k],dv)
			ret[k] = v

	return DictObject(**ret)
	



import requests

class NeedLogin(Exception):
	pass

class InsufficientPrivilege(Exception):
	pass

class HTTPError(Exception):
	def __init__(self,resp_code,url=None):
		self.resp_code = resp_code
		self.url = url
		Exception.__init__(self)

	def __expr__(self):
		return f'{self.url}:{self.resp_code}'

	def __str__(self):
		return f'{self.url}:{self.resp_code}'

hostsessions = {}

class Http_Client:
	def __init__(self):
		self.s = requests.Session()
		self.s.verify = False
		self.s.hooks['response'].append(self.response_handler)

	def prepped_handler(self, prepped):
		pass

	def response_handler(self, resp, *args, **kw):
		return resp
		
	def url2domain(self,url):
		parts = url.split('/')[:3]
		pre = '/'.join(parts)
		return pre

	def _webcall(self,url,method="GET",
					params={},
					files={},
					headers={},
					stream=False):
		domain = self.url2domain(url)
		sessionid = hostsessions.get(domain,None)
		if sessionid:
			headers.update({'session':sessionid})
		
		if method in ['GET']:
			req = requests.Request(method,url,
					params=params,headers=headers)
		else:
			req = requests.Request(method,url,
					data=params,files=files,headers=headers)
		prepped = self.s.prepare_request(req)
		self.prepped_handler(prepped)
		resp = self.s.send(prepped)
		if resp.status_code == 200:
			h = resp.headers.get('Set-Cookie',None)
			if h:
				sessionid = h.split(';')[0]
				hostsessions[domain] = sessionid

		if resp.status_code == 401:
			print('NeedLogin:',url)
			raise NeedLogin

		if resp.status_code == 403:
			raise InsufficientPrivilege

		if resp.status_code != 200:
			print('Error', url, method, 
					params, resp.status_code,
					type(resp.status_code))
			raise HTTPError(resp.status_code,url)
		return resp

	def webcall(self,url,method="GET",
				params={},
				files={},
				headers={},
				stream=False):
		resp = self._webcall(url,method=method,
				params=params,
				files=files,
				headers=headers,
				stream=stream)
		if stream:
			return resp
		try:
			data = resp.json()
			if type(data) != type({}):
				return data
			status = data.get('status',None)
			if status is None:
				return data
			if status == 'OK':
				return data.get('data')
			return data
		except:
			return resp.text
		
	def __call__(self,url,method="GET",
				params={},
				headers={},
				files={},
				stream=False):
		return self.webcall(url, method=method,
						params=params, files=files, 
						headers=headers, stream=stream)

	def get(self, url, params={}, headers={}, stream=False):
		return self.__call__(url,method='GET',params=params,
				headers=headers, stream=stream)
	def post(self, url, params={}, headers={}, files={}, stream=False):
		return self.__call__(url,method='POST',params=params, files=files,
				headers=headers, stream=stream)

	def put(self, url, params={}, headers={}, stream=False):
		return self.__call__(url,method='PUT',params=params,
				headers=headers, stream=stream)

	def delete(self, url, params={}, headers={}, stream=False):
		return self.__call__(url,method='DELETE',params=params,
				headers=headers, stream=stream)

	def option(self, url, params={}, headers={}, stream=False):
		return self.__call__(url,method='OPTION',params=params,
				headers=headers, stream=stream)
	



import json
from json import JSONEncoder
from inspect import ismethod, isfunction, isbuiltin, isabstract

def multiDict2Dict(md):
	ns = {}
	for k,v in md.items():
		ov = ns.get(k,None)
		if ov is None:
			ns[k] = v
		elif type(ov) == type([]):
			ov.append(v)
			ns[k] = ov
		else:
			ns[k] = [ov,v]
	return ns

class DictObject:
	def __init__(self,**kw):
		self.org_keys__ = []
		self.org_keys__ = [ k for k in self.__dict__.keys()]
		for k,v in kw.items():
			self.update({k:self.__DOitem(v)})
	
	def __getattr__(self,name):
		if name in self._addon().keys():
			return self.__getitem__(name)
		return None

	def update(self,kw):
		self.__dict__.update(kw)

	def _addon(self):
		ks = [ k for k in self.__dict__.keys() if k not in self.org_keys__]
		return {k:v for k,v in self.__dict__.items() if k in ks}

	def clear(self):
		for k in self._addon().keys():
			self.__dict__.pop(k)

	def get(self,name,default=None):
		return self._addon().get(name,default)

	def pop(self,k,default=None):
		return self.__dict__.pop(k,default)

	def popitem(self):
		return self.__dict__.popitem()

	def items(self):
		return self._addon().items()

	def keys(self):
		return self._addon().keys()

	def values(self):
		return self._addon().values()

	def __delitem__(self,key):
		self.pop(key)

	def __getitem__(self,name):
		return self._addon().get(name)
	
	def __setitem__(self,name,value):
		self.__dict__[name] = value

	def __str__(self):
		return str(self._addon())

	def __expr__(self):
		return self.addon().__expr__()

	def copy(self):
		return {k:v for k,v in self._addon().items()}

	def to_dict(self):
		d = self._addon()
		newd =  self.dict_to_dict(d)
		return newd

	def dict_to_dict(self,dic):
		d = {}	
		for k,v in dic.items():
			if isinstance(v,DictObject):
				d[k] = v.to_dict()
			elif isinstance(v,dict):
				d[k] = self.dict_to_dict(v)
			elif isinstance(v,list):
				d[k] = self.array_to_dict(v)
			elif k == '__builtins__':
				pass
			elif isbuiltin(v) or isfunction(v) or ismethod(v) or isabstract(v):
				pass
			else:
				d[k] = v
		return d

	def array_to_dict(self,v):
		r = []
		for i in v:
			if isinstance(i,list):
				r.append(self.array_to_dict(i))
			elif isinstance(i,dict):
				r.append(self.dict_to_dict(i))
			elif isinstance(i,DictObject):
				r.append(i.to_dict())
			elif isbuiltin(i) or isfunction(i) or ismethod(i) or isabstract(i):
				pass
			else:
				r.append(i)
		return r

	@classmethod
	def isMe(self,name):
		return name == 'DictObject'
		
	def __DOArray(self,a):
		b = [ self.__DOitem(i) for i in a ]
		return b
	
	def __DOitem(self, i):
		if isinstance(i,DictObject):
			return i
		if isinstance(i,dict):
			i = {k:v for k,v in i.items() if isinstance(k,str)}
			try:
				d = DictObject(**i)
				return d
			except Exception as e:
				print("****************",i,"*******dictObject.py")
				raise e
		if type(i) == type([]) or type(i) == type(()) :
			return self.__DOArray(i)
		return i

class DictObjectEncoder(JSONEncoder):
	def default(self, o):
		return o._addon()
		
	

def dictObjectFactory(_klassName__,**kwargs):
	def findSubclass(_klassName__,klass):
		for k in klass.__subclasses__():
			if k.isMe(_klassName__):
				return k
			k1 = findSubclass(_klassName__,k)
			if k1 is not None:
				return k1
		return None
	try:
		if _klassName__=='DictObject':
			return DictObject(**kwargs)
		k = findSubclass(_klassName__,DictObject)
		if k is None:
			return DictObject(**kwargs)
		return k(**kwargs)
	except Exception as e:
		print("dictObjectFactory()",e,_klassName__)
		raise e



# zmq_reqresp.py
import asyncio
import zmq
import zmq.asyncio
from .background import Background
from inspect import iscoroutinefunction

class ZmqRequester(object):
	def __init__(self, url, async_mode=False, timeout=0):
		super().__init__()
		self.async_mode = async_mode
		self.url = url
		self.timeout = timeout
		self._connect()

	def __del__(self):
		self._close()

	def _connect(self):
		if self.async_mode:
			self.ctx = zmq.asyncio.Context()
		else:
			self.ctx = zmq.Context()

		#  Socket to talk to server
		self.sock = self.ctx.socket(zmq.REQ)
		self.sock.connect(self.url)
		if self.timeout > 0:
			self.sock.setsockopt(zmq.LINGER, 0)
			self.poller = zmq.Poller()
			self.poller.register(self.sock, zmq.POLLIN)

	def _close(self):
		self.sock.close()
		self.ctx.term()

	def send(self, msg):
		"""
		send s string to responser, and return a string
		"""
		if self.async_mode:
			raise Exception('ZMQ_Requester: in async mode, use asend instead')
		b = msg.encode('utf-8')
		r = self.send_b(b)
		if r is not None:
			return r.decode('utf-8')
		return None

	def send_b(self, b):
		"""
		send a bytes and return a bytes
		"""
		if self.async_mode:
			raise Exception('ZMQ_Requester: in async mode, use asend_b instead')
		self.sock.send(b)
		if self.timeout > 0:
			if self.poller.poll(self.timeout * 1000):
				return self.sock.recv()
			else:
				self._close()
				self._connect()
				return None
		else:
			return self.sock.recv()

	async def asend_b(self, b):
		if not self.async_mode:
			raise Exception('ZMQ_Requester: not in async mode, use send_b instead')
		await self.sock.send_multipart([b])
		if self.timeout > 0:
			if self.poller.poll(self.timeout * 1000):
				r = await self.sock.recv_multipart()
				return r
			else:
				self._close()
				self._connect()
				return None
		r = await self.sock.recv_multipart()
		return r[0]

	async def asend(self, msg):
		if not self.async_mode:
			raise Exception('ZMQ_Requester: not in async mode, use send instead')
		b = msg.encode('utf-8')
		r = await self.asend_b(b)
		if r is None:
			return None
		return r.decode('utf-8')

class ZmqReplier(object):
	def __init__(self, url, handler, async_mode=False):
		self.async_mode = async_mode
		self.url = url
		if not self.async_mode and iscoroutinefunction(handler):
			raise('not in async mode, handler can not be a coroutine')

		self.handler = handler
		if self.async_mode:
			self.ctx = zmq.asyncio.Context()
		else:
			self.ctx = zmq.Context()
		self.sock = self.ctx.socket(zmq.REP)
		self.sock.bind(self.url)
		self.keep_running = True
		
	async def async_run(self):
		while self.keep_running:
			bs = await self.sock.recv_multipart()
			b = b[0]
			if iscoroutinefunction(self.handler):
				rb = await self.handler(b)
			else:
				rb =self.self.handler(b)

			if isinstance(rb, str):
				rb = rb.encode('utf-8')
			await self.sock.send_multipart([rb])

	def run(self):
		self.background = Background(self._run)
		self.background.daemon = True
		self.background.start()

	def _run(self):
		while self.keep_running:
			b = self.sock.recv()
			
			rb = self.handler(b)
			if isinstance(rb, str):
				rb = rb.encode('utf-8')
			self.sock.send(rb)

	def stop(self):
		self.keep_running = False
		self.join()



# Config.py
# Copyright (c) 2009 longtop Co.
# See LICENSE for details.
# author: yumoqing@gmail.com
# created date: 2009-02-01
# last modified date: 2009-02-05

import os,sys
from appPublic.ExecFile import ExecFile
from appPublic.dictObject import DictObject
from appPublic.Singleton import Singleton
from zope.interface import implements
CONFIG_FILE = 'conf/config.ini'
from folderUtils import ProgramPath
class Node(object) :
    pass

class Config:

    __metaclass = Singleton
    def __init__(self,configpath=None):
        if configpath is None:
            ps = CONFIG_FILE.split('/')
            configpath = os.path.join(ProgramPath(),*ps)
        self.configfile = configpath
        self.__execfile = ExecFile(self,path=configpath)
        self.__execfile.set('Node',Node)
        self.__execfile.set('DictObject',DictObject)
        self.__execfile.set('dict',DictObject)
        r,msg = self.__execfile.run()
        if not r:
            print(r,msg)

def getConfig(path=None):
    conf = Config(path)
    return conf
    



# -*- coding:UTF-8 -*-
from traceback import print_exc
from socket import *
import json
from appPublic.sockPackage import get_free_local_addr
from appPublic.background import Background
BUFSIZE = 1024 * 64
class UdpComm:
	def __init__(self, port, callback, timeout=1):
		self.callback = callback
		self.timeout = timeout
		self.host = get_free_local_addr()[0]
		self.port = port
		self.udpSerSock = socket(AF_INET, SOCK_DGRAM)
		# 设置阻塞
		self.udpSerSock.setblocking(1)
		# 设置超时时间 1s
		# self.udpSerSock.settimeout(timeout)
		self.udpSerSock.bind(('' ,port))
		self.run_flg = True
		self.thread = Background(self.run)
		self.thread.start()
	
	def run(self):
		while self.run_flg:
			try:
				b, addr = self.udpSerSock.recvfrom(BUFSIZE)
				if addr[0] != self.host:
					self.callback(b, addr)
			except Exception as e:
				print('exception happened:',e)
				print_exc()
				pass

	def stop(self):
		self.run_flg = False
		self.udpSerSock.close()
		
	def broadcast(self, data):
		broadcast_host = '.'.join(self.host.split('.')[:-1]) + '.255'
		udpCliSock = socket(AF_INET, SOCK_DGRAM)
		udpCliSock.settimeout(self.timeout)
		udpCliSock.bind(('', 0))  
		udpCliSock.setsockopt(SOL_SOCKET, SO_BROADCAST, 1)  
		b = data
		if not isinstance(data, bytes):
			b = json.dumps(data).encode('utf-8')
		udpCliSock.sendto(b, (broadcast_host,self.port))
	
	def send(self,data,addr):
		b = data
		if not isinstance(data, bytes):
			b = json.dumps(data).encode('utf-8')
		if isinstance(addr,list):
			addr = tuple(addr)
		self.udpSerSock.sendto(b,addr)

	def sends(self,data, addrs):
		b = data
		if not isinstance(data, bytes):
			b = json.dumps(data).encode('utf-8')
		for addr in addrs:
			if isinstance(addr,list):
				addr = tuple(addr)
			self.udpSerSock.sendto(b,addr)

if __name__ == '__main__':
	import sys
	def msg_handle(data, addr):
		print('addr:', addr, 'data=', data)

	port = 50000
	if len(sys.argv)>1:
		port = int(sys.argv[1])
	d = UdpComm(port, msg_handle)
	x = input()
	while x:
		d.broadcast(x)
		x = input()




import appPublic.timeUtils as tu 
import datetime as dt

class RestrictedEnv:
	def __init__(self):
		self.reg('today',self.today)
		self.reg('date',self.date)
		self.reg('datetime',self.datetime)
		self.reg('now',dt.datetime.now)
		
	def reg(self,k,v):
		self.__dict__[k] = v
		
	def run(self,dstr):
		dstr = '__tempkey__ = %s' % dstr
		exec(dstr,globals(),self.__dict__)
		return self.__tempkey__
		
	def today(self):
		now = dt.datetime.now()
		return tu.ymdDate(now.year,now.month,now.day)
		
	def date(self,dstr):
		return tu.str2Date(dstr)
	
	def datetime(self,dstr):
		return tu.str2Datetime(dstr)

if __name__ == '__main__':
	ns = RestrictedEnv()
	a = ns.run('today()')
	b = ns.run("date('2011-10-31')")
	c = ns.run('datetime("2012-03-12 10:22:22")')
	d = ns.run('now()')



# -*- coding = utf-8 -*-
ecc = { 
"Afghanistan":"阿富汗", 
"Aland Islands":"奥兰群岛", 
"Albania":"阿尔巴尼亚", 
"Algeria":"阿尔及利亚", 
"American Samoa":"美属萨摩亚", 
"Andorra":"安道尔", 
"Angola":"安哥拉", 
"Anguilla":"安圭拉", 
"Antigua and Barbuda":"安提瓜和巴布达", 
"Argentina":"阿根廷", 
"Armenia":"亚美尼亚", 
"Aruba":"阿鲁巴", 
"Australia":"澳大利亚", 
"Austria":"奥地利", 
"Azerbaijan":"阿塞拜疆", 
"Bangladesh":"孟加拉", 
"Bahrain":"巴林", 
"Bahamas":"巴哈马", 
"Barbados":"巴巴多斯", 
"Belarus":"白俄罗斯", 
"Belgium":"比利时", 
"Belize":"伯利兹", 
"Benin":"贝宁", 
"Bermuda":"百慕大", 
"Bhutan":"不丹", 
"Bolivia":"玻利维亚", 
"Bosnia and Herzegovina":"波斯尼亚和黑塞哥维那", 
"Botswana":"博茨瓦纳", 
"Bouvet Island":"布维岛", 
"Brazil":"巴西", 
"Brunei":"文莱", 
"Bulgaria":"保加利亚", 
"Burkina Faso":"布基纳法索", 
"Burundi":"布隆迪", 
"Cambodia":"柬埔寨", 
"Cameroon":"喀麦隆", 
"Canada":"加拿大", 
"Cape Verde":"佛得角", 
"Central African Republic":"中非", 
"Chad":"乍得", 
"Chile":"智利", 
"Christmas Islands":"圣诞岛", 
"Cocos (keeling) Islands":"科科斯（基林）群岛", 
"Colombia":"哥伦比亚", 
"Comoros":"科摩罗", 
"Congo (Congo-Kinshasa)":"刚果（金）", 
"Congo":"刚果", 
"Cook Islands":"库克群岛", 
"Costa Rica":"哥斯达黎加", 
"Cote D’Ivoire":"科特迪瓦", 
"China":"中国", 
"Croatia":"克罗地亚", 
"Cuba":"古巴", 
"Czech":"捷克", 
"Cyprus":"塞浦路斯", 
"Denmark":"丹麦", 
"Djibouti":"吉布提", 
"Dominica":"多米尼加", 
"East Timor":"东帝汶", 
"Ecuador":"厄瓜多尔", 
"Egypt":"埃及", 
"Equatorial Guinea":"赤道几内亚", 
"Eritrea":"厄立特里亚", 
"Estonia":"爱沙尼亚", 
"Ethiopia":"埃塞俄比亚", 
"Faroe Islands":"法罗群岛", 
"Fiji":"斐济", 
"Finland":"Finland", 
"France":"法国", 
"Franch Metropolitan":"法国大都会", 
"Franch Guiana":"法属圭亚那", 
"French Polynesia":"法属波利尼西亚", 
"Gabon":"加蓬", 
"Gambia":"冈比亚", 
"Georgia":"格鲁吉亚", 
"Germany":"德国", 
"Ghana":"加纳", 
"Gibraltar":"直布罗陀", 
"Greece":"希腊", 
"Grenada":"格林纳达", 
"Guadeloupe":"瓜德罗普岛", 
"Guam":"关岛", 
"Guatemala":"危地马拉", 
"Guernsey":"根西岛", 
"Guinea-Bissau":"几内亚比绍", 
"Guinea":"几内亚", 
"Guyana":"圭亚那", 
"Hong Kong":"香港 （中国）", 
"Haiti":"海地", 
"Honduras":"洪都拉斯", 
"Hungary":"匈牙利", 
"Iceland":"冰岛", 
"India":"印度", 
"Indonesia":"印度尼西亚", 
"Iran":"伊朗", 
"Iraq":"伊拉克", 
"Ireland":"爱尔兰", 
"Isle of Man":"马恩岛", 
"Israel":"以色列", 
"Italy":"意大利", 
"Jamaica":"牙买加", 
"Japan":"日本", 
"Jersey":"泽西岛", 
"Jordan":"约旦", 
"Kazakhstan":"哈萨克斯坦", 
"Kenya":"肯尼亚", 
"Kiribati":"基里巴斯", 
"Korea (South)":"韩国", 
"Korea (North)":"朝鲜", 
"Kuwait":"科威特", 
"Kyrgyzstan":"吉尔吉斯斯坦", 
"Laos":"老挝", 
"Latvia":"拉脱维亚", 
"Lebanon":"黎巴嫩", 
"Lesotho":"莱索托", 
"Liberia":"利比里亚", 
"Libya":"利比亚", 
"Liechtenstein":"列支敦士登", 
"Lithuania":"立陶宛", 
"Luxembourg":"卢森堡", 
"Macau":"澳门（中国）", 
"Macedonia":"马其顿", 
"Malawi":"马拉维", 
"Malaysia":"马来西亚", 
"Madagascar":"马达加斯加", 
"Maldives":"马尔代夫", 
"Mali":"马里", 
"Malta":"马耳他", 
"Marshall Islands":"马绍尔群岛", 
"Martinique":"马提尼克岛", 
"Mauritania":"毛里塔尼亚", 
"Mauritius":"毛里求斯", 
"Mayotte":"马约特", 
"Mexico":"墨西哥", 
"Micronesia":"密克罗尼西亚", 
"Moldova":"摩尔多瓦", 
"Monaco":"摩纳哥", 
"Mongolia":"蒙古", 
"Montenegro":"黑山", 
"Montserrat":"蒙特塞拉特", 
"Morocco":"摩洛哥", 
"Mozambique":"莫桑比克", 
"Myanmar":"缅甸", 
"Namibia":"纳米比亚", 
"Nauru":"瑙鲁", 
"Nepal":"尼泊尔", 
"Netherlands":"荷兰", 
"New Caledonia":"新喀里多尼亚", 
"New Zealand":"新西兰", 
"Nicaragua":"尼加拉瓜", 
"Niger":"尼日尔", 
"Nigeria":"尼日利亚", 
"Niue":"纽埃", 
"Norfolk Island":"诺福克岛", 
"Norway":"挪威", 
"Oman":"阿曼", 
"Pakistan":"巴基斯坦", 
"Palau":"帕劳", 
"Palestine":"巴勒斯坦", 
"Panama":"巴拿马", 
"Papua New Guinea":"巴布亚新几内亚", 
"Paraguay":"巴拉圭", 
"Peru":"秘鲁", 
"Philippines":"菲律宾", 
"Pitcairn Islands":"皮特凯恩群岛", 
"Poland":"波兰", 
"Portugal":"葡萄牙", 
"Puerto Rico":"波多黎各", 
"Qatar":"卡塔尔", 
"Reunion":"留尼汪岛", 
"Romania":"罗马尼亚", 
"Rwanda":"卢旺达", 
"Russian Federation":"俄罗斯联邦", 
"Saint Helena":"圣赫勒拿", 
"Saint Kitts-Nevis":"圣基茨和尼维斯", 
"Saint Lucia":"圣卢西亚", 
"Saint Vincent and the Grenadines":"圣文森特和格林纳丁斯", 
"El Salvador":"萨尔瓦多", 
"Samoa":"萨摩亚", 
"San Marino":"圣马力诺", 
"Sao Tome and Principe":"圣多美和普林西比", 
"Saudi Arabia":"沙特阿拉伯", 
"Senegal":"塞内加尔", 
"Seychelles":"塞舌尔", 
"Sierra Leone":"塞拉利昂", 
"Singapore":"新加坡", 
"Serbia":"塞尔维亚", 
"Slovakia":"斯洛伐克", 
"Slovenia":"斯洛文尼亚", 
"Solomon Islands":"所罗门群岛", 
"Somalia":"索马里", 
"South Africa":"南非", 
"Spain":"西班牙", 
"Sri Lanka":"斯里兰卡", 
"Sudan":"苏丹", 
"Suriname":"苏里南", 
"Swaziland":"斯威士兰", 
"Sweden":"瑞典", 
"Switzerland":"瑞士", 
"Syria":"叙利亚", 
"Tajikistan":"塔吉克斯坦", 
"Tanzania":"坦桑尼亚", 
"Taiwan":"台湾 （中国）", 
"Thailand":"泰国", 
"Trinidad and Tobago":"特立尼达和多巴哥", 
"Timor-Leste":"东帝汶", 
"Togo":"多哥", 
"Tokelau":"托克劳", 
"Tonga":"汤加", 
"Tunisia":"突尼斯", 
"Turkey":"土耳其", 
"Turkmenistan":"土库曼斯坦", 
"Tuvalu":"图瓦卢", 
"Uganda":"乌干达", 
"Ukraine":"乌克兰", 
"United Arab Emirates":"阿拉伯联合酋长国", 
"United Kingdom":"英国", 
"United States":"美国", 
"Uruguay":"乌拉圭", 
"Uzbekistan":"乌兹别克斯坦", 
"Vanuatu":"瓦努阿图", 
"Vatican City":"梵蒂冈", 
"Venezuela":"委内瑞拉", 
"Vietnam":"越南", 
"Wallis and Futuna":"瓦利斯群岛和富图纳群岛", 
"Western Sahara":"西撒哈拉", 
"Yemen":"也门", 
"Yugoslavia":"南斯拉夫", 
"Zambia":"赞比亚", 
"Zimbabwe":"津巴布韦" 
}

cec = {v:k for k,v in ecc.items()} 

def get_en_country_name(country):
	x = cec.get(country)
	if x:
		return x
	return country

def get_cn_country_name(country):
	x = ecc.get(country)
	if x:
		return x
	return country



import csv

class CSVData:
	def __init__(self,csvfile,names = None,headline = 0,dataline = 1):
		self.csvfile = csvfile
		self.names = names
		self.headline = headline
		self.dataline = dataline
	
	def read(self):
		f = open(self.csvfile,'rb')
		reader = csv.reader(f)
		fields = None
		if self.names is not None:
			fields = self.names
		data = []
		lno = 0
		for l in reader:
			if fields is None and lno == self.headline:
				fields = [f for f in l]
			if lno >= self.dataline:
				rec = {}
				for i in range(len(fields)):
					rec[fields[i]] = l[i]
				data.append(rec)
			lno += 1
		f.close()
		return data

	def iterRead(self):
		self.fd = open(self.csvfile,'r')
		try:
			reader = csv.reader(self.fd)
			fields = None
			if self.names is not None:
				fields = self.names
			lno = 0
			self.onBegin()
			for l in reader:
				if fields is None and lno == self.headline:
					fields = [f for f in l]
				if lno >= self.dataline:
					rec = {}
					for i in range(len(fields)):
						rec[fields[i]] = l[i]
					self.onRecord(rec)
				lno += 1
			self.fd.close()
			self.onFinish()
		except exception as e:
			fd.close()
			raise e

	def onReadBegin(self):
		pass
	def onRecord(self,rec):
		print(rec)
	
	def onFinish(self):
		print("onFinish() called")
		
if __name__ == '__main__':
	import sys
	cd = CSVData(sys.argv[1],names = ['st_date','open_price','max_price','min_price','close_price','volume','adj_price'])
	cd.iterRead()



#! /usr/bin/env python
import locale
import psutil
import socket


def getAllAddress():
	iocounts = psutil.net_io_counters(pernic=True)
	ns = [ k for k in iocounts.keys() if iocounts[k].bytes_sent>0 and iocounts[k].bytes_recv>0 ]
	stats = psutil.net_if_stats()
	stat = [ i for i in stats.keys() if i in ns ]
	hds = psutil.net_if_addrs()
	for n,v in hds.items():
		if n not in stat:
			continue
		for i in v:
			if i.family == socket.AF_INET:
				yield n,i.address
	
def getAllMacAddress():
	coding = locale.getdefaultlocale()[1]
	iocounts = psutil.net_io_counters(pernic=True)
	ns = [ k for k in iocounts.keys() if iocounts[k].bytes_sent>0 and iocounts[k].bytes_recv>0 ]
	stats = psutil.net_if_stats()
	stat = [ i for i in stats.keys() if i in ns ]
	hds = psutil.net_if_addrs()
	for n,v in hds.items():
		if n not in stat:
			continue
		for i in v:
			if i.family == socket.AF_PACKET:
				yield n,i.address

if __name__ == '__main__':
	def test():
		for i in getAllAddress():
			print("mac=",i)
	test()



# FiniteStateMachine.py
## a virtual State object of FSM
# 
class BaseFSM(object):
	def enterState(self, obj):
		raise NotImplementedError()
 
	def execState(self, obj):
		raise NotImplementedError()
 
	def exitState(self, obj):
		raise NotImplementedError()

## a FMS Manager
#  only need one Manager for a FSM
class FSMManager(object):
	def __init__(self):
		self._fsms = {}

	def addState(self,state,fsm):
		self._fsms[state] = fsm
	
	def delState(self,state):
		del self._fsms[state]      
	
	def getFSM(self, state):
		return self._fsms[state]
      
	def frame(self, objs, state):
		for obj in objs:
			if state == obj.curr_state:
				obj.keepState()
			else:
				obj.changeState(state, self._fsms[state])

## the object with has a Finite State Machine
# 
class FSMObject(object):
	def attachFSM(self,state,fsm):
		self.fsm_state_object = fsm
		self.fsm_cur_state = state
		
	def changeState(self,new_state,newfsm):
		self.fsm_cur_state = new_state
		self.fsm_state_object.exitState(self)
		self.fsm_state_object = new_fsm
		self.fsm_state_object.enterState(self)
		self.fsm_state_object.execState(self)
		
	def keepState(self):
		self.fsm_state_object.execState(self)
		
	


import codecs
import csv
class Reader:
	def __init__(self,f,delimiter):
		self.f = f
		self.delimiter = delimiter
		self.line = 0

	def __iter__(self):
		return self
	
	def next(self):
		l = self.f.readline()
		if l == '':
			raise StopIteration()
		while l[-1] in [ '\n','\r']:
			l = l[:-1]
		r = [ i if i != '' else None for i in l.split(self.delimiter) ]
		self.line = self.line + 1
		return r

class CSVData:
	def __init__(self,filename,coding='utf8',delimiter=','):
		self.filename = filename
		self.coding = coding
		self.f = codecs.open(filename,'rb',self.coding)
		self.reader = Reader(self.f,delimiter)
		self.fields = self.reader.next()
	
	def __del__(self):
		self.f.close()
	
	def __iter__(self):
		return self
	
	def next(self):
		try:
			r = self.reader.next()
			if len(r) != len(self.fields):
				print("length diff",len(r),len(self.fields),"at line %d" % self.reader.line)
				raise StopIteration()
			d = {}
			[d.update({self.fields[i]:r[i]}) for i in range(len(self.fields))]
			return d
		except:
			raise StopIteration()

if __name__ == '__main__':
	import sys
	cd = CSVData(sys.argv[1])
	for r in cd:
		print(r)
	




# !/usr/bin/env python
# -*- coding:UTF-8 -*-
from socket import *
import json
from appPublic.sockPackage import get_free_local_addr
from appPublic.background import Background
BUFSIZE = 1024
class BroadcastServer:
	def __init__(self, port, info):
		self.info = info
		self.port = port
		self.udpSerSock = socket(AF_INET, SOCK_DGRAM)
		# 设置阻塞
		self.udpSerSock.setblocking(1)
		# 设置超时时间 1s
		# self.udpSerSock.settimeout(1)
		self.udpSerSock.bind(('' ,port))
		self.run_flg = True
		self.thread = Background(self.run)
		self.thread.start()
	
	def run(self):
		while self.run_flg:
			try:
				data, addr = self.udpSerSock.recvfrom(BUFSIZE)
				ret = json.dumps(self.info).encode('utf-8')
				self.udpSerSock.sendto(ret, addr)
			except Exception as e:
				print('exception happened:',e)
				pass

	def stop(self):
		self.run_flg = False
		self.udpSerSock.close()
		
def find_players(port):
	# broadcast_addr = '.'.join(host.split('.')[:-1]) + '.255'
	host = get_free_local_addr()[0]
	udpCliSock = socket(AF_INET, SOCK_DGRAM)
	#设置阻塞
	#udpCliSock.setblocking(2)
	#设置超时时间
	udpCliSock.settimeout(5)
	udpCliSock.bind(('', 0))  
	udpCliSock.setsockopt(SOL_SOCKET, SO_BROADCAST, 1)  
	udpCliSock.sendto(b'findplayers', ('255.255.255.255',port))
	players = []
	while True:
		try:
			data,addr = udpCliSock.recvfrom(BUFSIZE)
			if  addr[0] != host and data:
				data = data.decode('utf-8')
				d = json.loads(data)
				d['ip'] = addr[0]
				players.append(d)
		except Exception as e:
			break
	udpCliSock.close()
	return players




import pickle

def saveData(fn,*args):
  f = open(fn,'wb')
  a = [ pickle.dump(arg,f) for arg in args ]
  f.close()
  
def loadData(fn,cnt):
  a = [None] * cnt
  try:
    f = open(fn,'rb')
    a = [ pickle.load(f) for i in range(cnt) ]
    f.close()
    return a
  except:
    return a



import os
import time
import threading
import sys
import socket

def get_free_local_addr():
	with socket.socket(socket.AF_INET, socket.SOCK_DGRAM) as s:
		s.connect(("8.8.8.8", 80))
		return s.getsockname()
		# return (ip,port)

class background(threading.Thread) :
        def __init__(self,func,kw) :
                threading.Thread.__init__(self)
                self.func = func
                self.kw = kw

        def run(self) :
                if self.func!=None :
                        self.func(**self.kw)
                        return

def BackgroundCall(func,datas) :
        b=background(func,datas)
        b.start()
        return

class SocketServerError(Exception) :
	pass

class SocketClientError(Exception) :
	pass

class SocketServer(threading.Thread) :

	def __init__(self,host,port,max_connect=10,callee=None) :
		threading.Thread.__init__(self, name = 'SocketServer')
		self.setDaemon(False)
		self.host = host
		self.port = int(port)
		self.max_c = max_connect
		self.ready = False
		self.keep_running = 0
		self.callee = callee
		self.setSocketServer()

	def setSocketServer(self) :
		try :
			self.sock = socket.socket(socket.AF_INET,socket.SOCK_STREAM)
			self.sock.bind((self.host,self.port))
			self.sock.listen(self.max_c)
			self.ready = True
		except Exception as e:
			print('setSocketServer() Error:%s\nhost=%s,port=%d' % (e,self.host,self.port))
			pass

	def run(self) :
		if not self.ready :
			raise SocketServerError('not ready')
		callee = self.callee
		if self.callee!=None :
			callee = self.callee
		self.keep_running = 1
		while self.keep_running :
			conn,addr = self.sock.accept()
			BackgroundCall(callee,{'conn':conn,'addr':addr})
			# conn.close()
	
	def stop(self) :
		self.keep_running = 0

	def callee(self,conn,addr) :
		while 1 :
			d = conn.recv(1024)
			if d==None :
				break
			conn.send(d)
		con.close()

class SocketClient :

	def __init__(self,host,port) :
		self.host = host
		self.port = port
		self.ready = False
		self.connect()

	# if tim ==0 not blocking
	def timeout(self,tim) :
		if self.ready :
			self.sock.setblocking(tim>0)
			if tim>0 :
				self.sock.settimeout(tim)

	def connect(self) :
		try :
			self.sock = socket.socket(socket.AF_INET,socket.SOCK_STREAM)
			self.sock.connect((self.host,self.port))
			self.ready = True
		except Exception as e:
			self.ready = False
			print('Socket connect error,%s\nhost=%s,port=%s' % (e,self.host,self.port))
			raise SocketClientError('connect error')

	def read(self,size) :
		try :
			data = self.sock.recv(size)
			return data
		except Exception as e:
			print('recv error,%s' % e)
			raise SocketClientError('recv error')

	def write(self,data) :
		try :
			self.sock.send(data)
		except Exception as e:
			print('recv error,%s' % e)
			raise SocketClientError('send error')
	
	def close(self) :
		self.sock.close()
		self.ready = False

if __name__ == '__main__' :
	s = SocketServer('localhost',12232)
	s.start()
	time.sleep(5)
	while 1 :
		c = SocketClient('localhost',12232)
		msg = 'msg1'
		print("send:",msg)
		c.write(msg)
		d = c.read(1024)
		print("get:",d)
		time.sleep(1)



import time
from multiprocessing import Process
import threading
import random
from appPublic.background import Background

class ProcessWorkers:
	def __init__(self, worker_cnt=10):
		self.semaphore = threading.Semaphore(value=worker_cnt)
		self.co_worker = 0
	def _do(self, func, *args, **kwargs):
		self.semaphore.acquire()
		self.co_worker += 1
		p = Process(target=func, args=args, kwargs=kwargs)
		p.start()
		p.join()
		self.co_worker -= 1
		self.semaphore.release()

	def do(self, func, *args, **kwargs):
		b = Background(self._do, func, *args, **kwargs)
		b.start()

	def get_workers(self):
		return self.co_worker

if __name__ == '__main__':
	def k(worker):
		t = random.randint(1,4)
		print('current workers=',worker.get_workers(), 'sleep=', t)
		time.sleep(t)

	w = ProcessWorkers()
	for i in range(100):
		w.do(k, w)




import rsa

class RSA:
	
	def __init__(self, keylength=4096, coding='iso8859'):
		self.coding = coding
		self.keylength = keylength
		
	def write_privatekey(self,private_key,fname,password=None):
		bd = private_key.save_pkcs1()
		with open(fname, 'wb') as f:
			f.write(bd)

	def publickeyText(self,public_key):
		bd = public_key.save_pkcs1()
		return bd.decode(self.coding)

	def write_publickey(self,public_key,fname):
		bd = public_key.save_pkcs1()
		with open(fname, 'wb') as f:
			f.write(bd)

	def read_privatekey(self,fname,password=None):
		with open(fname, 'rb') as pf:
			kd = pf.read()
			return rsa.PrivateKey.load_pkcs1(kd)

	def publickeyFromText(self,text):
		bd = text.encode(self.coding)
		rsa.PublicKey.load_pkcs1(bd)

	def read_publickey(self,fname):
		with open(fname, 'rb') as pf:
			kd = pf.read()
			return rsa.PublicKey.load_pkcs1(kd)

	def create_privatekey(self, keylength=4096):
		_, prik = rsa.newkeys(keylength)
		return prik

	def create_publickey(self,private_key):
		return rsa.PublicKey(private_key.n, private_key.e)

	def encode_bytes(self, public_key, bdata):
		return rsa.encrypt(bdata, public_key)

	def encode(self,public_key,text):
		bdata = text.encode(self.coding)
		bc = self.encode_bytes(public_key, bdata)
		return bc.decode(self.coding)

	def decode_bytes(self, private_key, bdata):
		return rsa.decrypt(bdata, private_key)

	def decode(self,private_key,cipher):
		bc = cipher.encode(self.coding)
		bd = self.decode_bytes(private_key, bc)
		return bd.decode(self.coding)

	def sign_bdata(self, private_key, data_to_sign):
		return rsa.sign(data_to_sign, private_key, 'SHA-1')

	def sign(self,private_key,message):
		bd = message.encode(self.coding)
		bs = self.sign_bdata(private_key, bd)
		return bs.decode(self.coding)

	def check_sign_bdata(self, public_key, bdata, sign):
		try:
			r = rsa.verify(bdata, sign, public_key)
			if r == 'SHA-1':
				return True
		except:
			return False


	def check_sign(self,public_key,plain_text,signature):
		bd = plain_text.encode(self.coding)
		bs = signature.encode(self.coding)
		return self.check_sign_bdata(public_key, bd, bs)


if __name__ == '__main__':
	import os
	prikey1_file = os.path.join(os.path.dirname(__file__),'..','test', 'prikey1.rsa')
	r = RSA()
	mpri = r.create_privatekey(2048)
	mpub = r.create_publickey(mpri)
	
	zpri = r.create_privatekey(2048)
	zpub = r.create_publickey(zpri)
	
	l = 100
	while True:
		text = 'h' * l
		cipher = r.encode(mpub,text)
		ntext = r.decode(mpri,cipher)
		print('textlen=', l, 'encode text=', text, \
				'decode result=', ntext,
				'cyber size=', len(cipher),
				'check if equal=', text==ntext)
		signature = r.sign(zpri,text)
		check = r.check_sign(zpub,text,signature)
		print('sign and verify=',len(signature),check)
		l += 1



import sys
import threading
from threading import Thread
from queue import Queue, Empty

class Worker(Thread):
	def __init__(self, rqueue, timeout=1):
		Thread.__init__(self)
		self.timeout = timeout
		self.setDaemon(False)
		self.r_queue = rqueue
		self.start()

	def run(self):
		emptyQueue = False
		while True:
			try:
				callable,args,kw = self.r_queue.get(timeout=self.timeout)
				if callable is None:
					break
				callable(*args,**kw)

			except Empty:
				time.sleep(1)

	def resulthandler(self,rez):
		pass

class ThreadWorkers:
	def __init__(self,num_workers=20):
		self.workQueue = Queue()
		self.worker_cnt = num_workers
		self.workers = []
		self.__createThreadPool(num_workers)

	def __createThreadPool(self,num):
		for i in range(num):
			thread = Worker(self.workQueue)
			self.workers.append(thread)
	
	def wait_for_complete(self):
		for i in range(self.worker_cnt):
			self.add_job(None,None,None)

		while len(self.workers):
			thread = self.workers.pop()
			if thread.isAlive():
				thread.join()
	
	def add_job(self,callable,args=[],kw={}):
		self.workQueue.put([callable,args,kw])
if __name__ == '__main__':
	import requests
	def get(url):
		x = requests.get(url)
		print(x.status_code)

	tw = ThreadWorkers()
	for i in range(10000):
		tw.add_job(get,['http://www.baidu.com'])
	tw.wait_for_complete()
	print('finished')



import time
import datetime
from .Singleton import SingletonDecorator

timerecord = {}

class TimeCost:
	def __init__(self,name):
		self.name = name
	
	def __enter__(self):
		self.begin_time = time.time()
	
	def __exit__(self,*args):
		self.end_time = time.time()
		d = timerecord.get(self.name,[])
		d.append(self.end_time - self.begin_time)
		timerecord[self.name] = d

	def clear(self):
		timerecord = {}

	@classmethod
	def clear_all(self):
		timerecord = {}

	@classmethod
	def clear(self, name):
		timerecord[name] = []

	@classmethod
	def show(self):
		def getTimeCost(name):
			x = timerecord.get(name,[])
			if len(x) == 0:
				return 0,0,0
			return len(x), sum(x), sum(x)/len(x)
		
		print('TimeCost ....')
		for name in timerecord.keys():
			print(name, * getTimeCost(name))





def calculate_luminence(rgba):
	return 0.2126 * color[0] + \
			0.7152 * color[1] + \
			0.0722 * colr[2]

def get_contrast_ratio(lumA, lumB):
	lighter = max(lumA, lumB)
	darker = min(lumX, lumB)
	return (lighter + 0.05) / (darker + 0.05)

def get_color_contrast_ratio(color1, color2):
	lum1 = calculate_luminence(color1)
	lum2 = calculate_luminence(color2)
	return get_contrast_Ratio(lum1, lum2)
	
def wcag_check(color1, color2, font_size=14):
	aa = 3.0
	aaa = 4.5
	if font_size < 18:
		aa = 4.5
		aaa = 7.0
	ratio = get_color_contrast_ratio(color1, color2)
	return ratio >= aa, radio >= aaa

if __name__ == '__main__':
	pass



import os,re,sys
import codecs
from appPublic.folderUtils import _mkdir
from appPublic.Singleton import SingletonDecorator
from appPublic.folderUtils import ProgramPath
from appPublic.jsonConfig import getConfig
import threading
import time

import locale

comment_re = re.compile(r'\s*#.*')
msg_re = re.compile(r'\s*([^:]*)\s*:\s*([^\s].*)')

def dictModify(d, md) :
	for i in md.keys() :
		if md[i]!=None :
			d[i] = md[i]
	return d

convert_pairs = 	{':':'\\x3A',
	'\n':'\\x0A',
	'\r':'\\x0D',
}

def charEncode(s) :
	r = ''
	v = s.split('\\')
	s = '\\\\'.join(v)
	for i in convert_pairs.keys() :
		v = s.split(i)
		s = convert_pairs[i].join(v)
		# print 'i=',i,'iv=',convert_pairs[i],'s=',s
	return s
 
def charDecode(s) :
	for i in convert_pairs.items() :
		v = s.split(i[1])
		s = i[0].join(v)
	v = s.split('\\\\')
	s = '\\'.join(v)
	return s
     
def getTextDictFromLines(lines) :
	d = {}
	for l in lines :
		l = ''.join(l.split('\r'))
		if comment_re.match(l) :
			continue
		m = msg_re.match(l)
		if m :
			grp = m.groups()
			d[charDecode(grp[0])] = charDecode(grp[1])
	return d

def getFirstLang(lang) :
	s = lang.split(',')
	return s[0]

@SingletonDecorator
class MiniI18N:
	"""
	"""
	def __init__(self,path,lang=None,coding='utf8') :
		self.path = path
		l = locale.getdefaultlocale()
		self.curLang = l[0]
		self.coding = coding
		self.id = 'i18n'
		self.langTextDict = {}
		self.messages = {}
		self.setupMiniI18N()
		self.missed_pt = None
		self.translated_pt = None
		self.header_pt = None
		self.footer_pt = None
		self.show_pt=None
		self.clientLangs = {}
		self.languageMapping = {}
		self.timeout = 600
		config = getConfig()
		for l1,l in config.langMapping.items():
			self.setLangMapping(l1,l)
	
	def __call__(self,msg,lang=None) :
		"""
		"""
		if type(msg) == type(b''):
			msg = msg.decode(self.coding)
		return self.getLangText(msg,lang)
		
	def setLangMapping(self,lang,path):
		self.languageMapping[lang] = path
		
	def getLangMapping(self,lang):
		return self.languageMapping.get(lang,lang)

	def setTimeout(self,timeout=600):
		self.timeout = timeout
	
	def delClientLangs(self):
		t = threading.currentThread()
		tim = time.time() - self.timeout
		[ self.clientLangs.pop(k,None) for k in self.clientLangs.keys() if self.clientLangs[k]['timestamp'] < tim ]
				
	def getLangDict(self,lang):
		lang = self.getLangMapping(lang)
		return self.langTextDict.get(lang,{})
		
	def getLangText(self,msg,lang=None) :
		"""
		"""
		if lang==None :
			lang = self.getCurrentLang()
		textMapping = self.getLangDict(lang)
		return textMapping.get(msg,msg)

	def setupMiniI18N(self) :
		"""
		"""

		p = os.path.join(self.path,'i18n')
		langs = []
		
		for f in os.listdir(p) :
			if os.path.isdir(os.path.join(p,f)) :
				langs.append(f)
		for dir in langs :
			p1 = os.path.join(p,dir,'msg.txt')
			if os.path.exists(p1) :
				f = codecs.open(p1,'r',self.coding)
				textDict = getTextDictFromLines(f.readlines())
				f.close()
				d = {}
				if dir in self.langTextDict :
					d = self.langTextDict[dir]
				self.langTextDict[dir] = textDict
				for i in textDict.keys() :
					self.messages[i] = ''
				
		self._p_changed = 1
		
	def setCurrentLang(self,lang):
		lang = self.getLangMapping(lang)
		t = time.time()
		threadid = threading.currentThread()
		a = dict(timestamp=t,lang=lang)
		self.clientLangs[threadid] = a

	def getCurrentLang(self) :
		"""
		"""
		threadid = threading.currentThread()
		return self.clientLangs[threadid]['lang']

def getI18N(path=None, coding='utf8'):
	if path is None:
		path = ProgramPath()
	i18n = MiniI18N(path,coding)
	return i18n




import poplib,pdb,email,re,time
from email import header
import datetime
import os

POP_ADDR = r'pop.126.com'
USER = ''
PASS = ''
CONFIG = ''

def getYear(date):
	rslt = re.search(r'\b2\d{3}\b', date)
	return int(rslt.group())

def getMonth(date):
	monthMap = {'Jan':1,'Feb':2,'Mar':3,'Apr':4,'May':5,'Jun':6,
				'Jul':7,'Aug':8,'Sep':9,'Oct':10,'Nov':11,'Dec':12,}

	rslt = re.findall(r'\b\w{3}\b', date)
	for i in range(len(rslt)):
		month = monthMap.get(rslt[i])
		if None != month:
			break

	return month

def getDay(date):
	rslt = re.search(r'\b\d{1,2}\b', date)
	return int(rslt.group())

def getTime(date):
	rslt = re.search(r'\b\d{2}:\d{2}:\d{2}\b', date)
	timeList = rslt.group().split(':')

	for i in range(len(timeList)):
		timeList[i] = int(timeList[i])

	return timeList

def transformDate(date):
	rslt = getYear(date)
	rslt = rslt * 100
	rslt = rslt + getMonth(date)
	rslt = rslt * 100
	rslt = rslt + getDay(date)
	   

	timeList = getTime(date)
	for i in range(len(timeList)):
		rslt = rslt * 100
		rslt = rslt + timeList[i]

	return rslt

def getRecentReadMailTime():
	fp = open(CONFIG, 'r')
	rrTime = fp.read()
	fp.close()
	return rrTime

def setRecentReadMailTime():
	fp = open(CONFIG, 'w')
	fp.write(time.ctime())
	fp.close()
	return

def getTimeEarly(period):
	def years(n):
		return datetime.timedelta(years=n)
	def months(n):
		return datetime.timedelta(years=n)
	def days(n):
		return datetime.timedelta(days=n)
	def hours(n):
		return datetime.timedelta(hours=n)
	def minutes(n):
		return datetime.timedelta(minutes=n)
	def seconds(n):
		return datetime.timedelta(seconds=n)
	
	funcs={
		'y':years,
		'm':months,
		'd':days,
		'H':hours,
		'M':minutes,
		'S':seconds,
	}
	pattern='(\d*)([ymdHMS])'
	r=re.compile(pattern)
	s = r.findall(period)
	t = datetime.datetime.now()
	for v,ty in s:
		td = funcs[ty](int(v))
		t = t - td
	return time.ctime(t.timestamp())
	

def parseMailContent(msg):
	if msg.is_multipart():
		for part in msg.get_payload():
			parseMailContent(part)
	else:
		bMsgStr = msg.get_payload(decode=True)
		charset = msg.get_param('charset')
		msgStr = 'Decode Failed'
		try:
			if None == charset:
				msgStr = bMsgStr.decode()
			else:
				msgStr = bMsgStr.decode(charset)
		except:
			pass
		
		print(msgStr)

def recvEmail(POP_ADDR,USER,PASS,PERIOD,callback):
	server = poplib.POP3(POP_ADDR)
	server.user(USER)
	server.pass_(PASS)

	mailCount,size = server.stat()
	mailNoList = list(range(mailCount))
	mailNoList.reverse()
	FROMTIME = getTimeEarly(PERIOD)
	hisTime = transformDate(FROMTIME)
	#pdb.set_trace()
	for i in mailNoList:
		message = server.retr(i+1)[1]
		mail = email.message_from_bytes(b'\n'.join(message))

		if transformDate(mail.get('Date')) > hisTime:
			if not callback(mail):
				break
			#parseMailContent(mail)
		else:
			break



import sys
import zmq
import time
from zmq import Context
from appPublic.jsonConfig import getConfig

class TopicServer:
	def __init__(self, address='127.0.0.1', pub_port='5566', sub_port='5567'):
		# get ZeroMQ version
		print("Current libzmq version is %s" % zmq.zmq_version())
		print("Current  pyzmq version is %s" % zmq.pyzmq_version())

		self.context = Context.instance()
		# 2 sockets, because we can only bind once to a socket (as opposed to connect)
		self.pub_port = "tcp://{}:{}".format(address, pub_port)
		self.sub_port = "tcp://{}:{}".format(address, sub_port)

		self.xpub_xsub_proxy()

	# N publishers to 1 sub; proxy 1 sub to 1 pub; publish to M subscribers
	def xpub_xsub_proxy(self):
		print("Init proxy")

		# Socket subscribing to publishers
		frontend_pubs = self.context.socket(zmq.XSUB)
		frontend_pubs.bind(self.pub_port)

		# Socket publishing to subscribers
		backend_subs = self.context.socket(zmq.XPUB)
		backend_subs.bind(self.sub_port)

		print("Try: Proxy... CONNECT!")
		zmq.proxy(frontend_pubs, backend_subs)
		print("CONNECT successful!")
		"""
		while True:
			time.sleep(1)
		"""

class ConfiguredTopicServer(TopicServer):
	"""
	in config file has a topicserver key
	{
		"topicserver":{
			"address":"11.11.1.11",
			"pub_port":1234,
			"sub_server":1235
		}
	}
	"""
	def __init__(self):
		config = getConfig()
		params = config.topicserver
		if not params:
			raise MissTopicServerConfig
		super(ConfiguredTopicServer, self).__init__(**params)

class TopicPublisher:
	def __init__(self, topic='en', address='127.0.0.1', port='5566'):
		# get ZeroMQ version
		print("Current libzmq version is %s" % zmq.zmq_version())
		print("Current  pyzmq version is %s" % zmq.pyzmq_version())

		self.topic = topic
		self._topic = topic.encode('utf-8')
		self.context = Context.instance()
		self.url = "tcp://{}:{}".format(address, port)
		self.pub = self.context.socket(zmq.PUB)
		self.pub.connect(self.url)
		time.sleep(0.5)

	def send(self, message):
		self.pub.send_multipart([self._topic, message.encode('utf-8')])

class ConfiguredTopicPublisher(TopicPublisher):
	def __init__(self, topic=''):
		config = getConfig()
		params = config.topicserver
		if not params:
			raise MissTopicServerConfig
		super(ConfiguredTopicPublisher, self).__init__(topic=topic, 
					address = params.address,
					port=params.pub_port)
		
class TopicSubscriber:
	def __init__(self, topic='', address='127.0.0.1', port='5567', callback=None):
		# get ZeroMQ version
		print("Current libzmq version is %s" % zmq.zmq_version())
		print("Current  pyzmq version is %s" % zmq.pyzmq_version())

		self.callback = callback
		self.topic = topic
		self.context = Context.instance()
		self.url = "tcp://{}:{}".format(address, port)
		self.sub = self.context.socket(zmq.SUB)
		self.sub.connect(self.url)
		# subscribe to topic 'en' or 'jp'
		if isinstance(self.topic, list):
			for t in self.topic:
				self.sub.setsockopt(zmq.SUBSCRIBE, t.encode('utf-8'))
		else:
			self.sub.setsockopt(zmq.SUBSCRIBE, self.topic.encode('utf-8'))


	def run(self):
		# keep listening to all published message, filtered on topic
		print("Sub {}: Going to wait for messages!".format(self.topic))
		while True:
			msg_received = self.sub.recv_multipart()
			print("sub {}: {}".format(self.topic, msg_received))
			if self.callback:
				self.callback(msg_received)

class ConfiguredTopicSubscriber(TopicSubscriber):
	def __init__(self, topic=''):
		config = getConfig()
		params = config.topicserver
		if not params:
			raise MissTopicServerConfig
		super(ConfiguredTopicSubscriber, self).__init__(topic=topic, 
						address=params.address,
						port=params.sub_port)
		



import os
import sys
try:
	import ujson as json
except:
	import json
from jinja2 import Environment,FileSystemLoader, BaseLoader
import codecs
from appPublic.argsConvert import ArgsConvert
from appPublic.dictObject import DictObject
def isNone(obj):
	return obj is None


def string_template_render(tmp_string, data):
	rtemplate = Environment(loader=BaseLoader()).from_string(tmp_string)
	return rtemplate.render(**data)

class MyTemplateEngine:
	def __init__(self,pathList,file_coding='utf-8',out_coding='utf-8'):
		self.file_coding = file_coding
		self.out_coding = out_coding
		loader = FileSystemLoader(pathList, encoding=self.file_coding)
		self.env = Environment(loader=loader, enable_async=False)	
		denv={
			'json':json,
			'hasattr':hasattr,
			'int':int,
			'float':float,
			'str':str,
			'type':type,
			'isNone':isNone,
			'len':len,
			'render':self.render,
			'renders':self.renders,
			'ArgsConvert':ArgsConvert,
			'renderJsonFile':self.renderJsonFile,
			'ospath':lambda x:os.path.sep.join(x.split(os.altsep)),
			'basename':lambda x:os.path.basename(x),
			'basenameWithoutExt':lambda x:os.path.splitext(os.path.basename(x))[0],
			'extname':lambda x:os.path.splitext(x)[-1],
		}
		self.env.globals.update(denv)

	def set(self,k,v):
		self.env.globals.update({k:v})
		
	def _render(self,template,data):
		# print('**********template=',template,'**data=',data,'type_data=',type(data),'************')
		uRet = template.render(**data)
		return uRet
		
	def renders(self,tmplstring,data):
		def getGlobal():
			return data
		self.set('global',getGlobal)
		template = self.env.from_string(tmplstring)
		return self._render(template,data)

	def render(self,tmplfile,data):
		def getGlobal():
			return data
		self.set('global',getGlobal)
		template = self.env.get_template(tmplfile)
		return self._render(template,data)

	def renderJsonFile(self,tmplfile,jsonfile):
		with codecs.open(jsonfile,"r",self.file_coding) as f:
			data = json.load(f)
			return self.render(tmplfile,data)



#
from appPublic.dictObject import DictObject

class SingletonDecorator:
    def __init__(self,klass):
        self.klass = klass
        self.instance = None
    def __call__(self,*args,**kwds):
        if self.instance == None:
            self.instance = self.klass(*args,**kwds)
        return self.instance

@SingletonDecorator
class GlobalEnv(DictObject):
	pass

if __name__ == '__main__':
	@SingletonDecorator
	class Child(object):
		def __init__(self,name):
			print("clild.init")
			self.name = name
		def __str__(self):
			return 'HAHA：' + self.name
		def __expr__(self):
			print(self.name)
        
	@SingletonDecorator
	class Handle(object):
		def __init__(self,name):
			self.name = name
		def __expr__(self):
			print(self.name)
  
	c = Child('me')
	d = Child('he')
	
	print(str(c),str(d))
	e = Handle('hammer')
	f = Handle('nail');
	print(str(e),str(f))
	



# -*- coding=utf-8 -*-

"""
kivy color:
[ r, g, b, a]
不同的颜色值总能找到一个人眼感知的灰度值，这是著名的心理学公式：
灰度 = 红×0.299 + 绿×0.587 + 蓝×0.114
当灰度值大于0.5时使用暗色，否则使用明色
colors 两个颜色，缺省为空，使用函数内置的两个颜色
"""

def color_gray_rate(color):
	graylevel = 0.299 * color[0] + \
				0.587 * color[1] + \
				0.114 * color[2]
	return graylevel

def get_fgcolor_from_bgcolor(bgcolor, colors=None):
	dark_fgcolor=[0.11,0.11,0.11,1]
	bright_fgcolor=[0.89,0.89,0.89,1]
	
	graylevel = color_gray_rate(bgcolor)
	if colors == None:
		if graylevel > 0.5:
			return dark_fgcolor
		else:
			return bright_fgcolor
	r1 = color_gray_rate(colors[0])
	r2 = color_gray_rate(colors[1])
	if abs(graylevel - r1) > abs(graylevel - r2):
		return colors[0]
	return colors[1]

